/***************************************************************************
 *
 * File name               :browser.c
 * 
 * Description             :New conference manager
 *
 * Authors                 :Janusz Bulawa & Tomasz Stachowiak, NPAC
 *
 * Date                    :10-07-96
 *
 ***************************************************************************/

#define XP_UNIX

#include <Xm/Xm.h>
#include "launch.c"
#include "npapi.h"
#include <stdio.h>
#include <stdlib.h>

#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <Xm/DialogS.h>
#include <X11/Shell.h>
#include <Xm/MessageB.h>
#include <Xm/List.h>

/* Motif resource manager (for .uid interface files)*/
#include <Mrm/MrmPublic.h>

#define MIME_EXT_DESCRIPTION "application/x-dve:.dve"
/* In case of stupid server "text/plain:.dve" */

typedef struct _PluginInstance
{
    Widget	netscape_widget; /* The widget provided by netscape */
    Display    *display;         /* Netscape's display connection */
    uint16	mode;            /* NP_EMBED, NP_FULL, or NP_BACKGROUND */
    uint32 width, height;
    
    int len;
    XtIntervalId timerId;
    XtAppContext appContext;
} PluginInstance;



XtAppContext    AppContext;
Widget          parent;
Widget          pluginsLabel, membersLabel, pluginsList, membersAccList;
XmStringTable   plugTlist;
XtAppContext	appMem;

/* Add member dialog widgets */
Widget	 mainWin, mainForm, textLine, bMemOK, bMemCancel,
         usr_text, host_text;


/* my indicator of existing conference */
int conferenceExist = 0;

Arg wArg[10];
int numArg;
XmString label;
int err;
int create_recipient_list;

int connectDVE();
static void manage(void);
static void addMemOk();
static void addMemCancel();
static void partyAccepted(DveSession dveSession, DveEvent* event);
static void partyRejected(DveSession dveSession, DveEvent* event);
static void partyLeft(DveSession dveSession, DveEvent* event);
static void partyNew(DveSession dveSession, DveEvent* event);
static void changeDND(DveSession dveSession, DveEvent* event);
static void SetMessage(const char *msg);

int introduceYourself(DveSession *session,
		      DveSession *privatesession);
int init_plugins(DveSession session);
void     PluginReady(pluginObject plugin);
void     PluginDone(pluginObject plugin);
int init_members(DveSession session);
void parseInformation(Widget list, char *info);
/* 
 * Main panel buttons callbacks 
 */
static void add_member(Widget w, XtPointer data1, XtPointer data2);
static void info_plugin(Widget w, XtPointer data1, XtPointer data2);
static void show_plugin(Widget w, XtPointer data1, XtPointer data2);
static void hide_plugin(Widget w, XtPointer data1, XtPointer data2);
static void accept_invitation(Widget w, XtPointer data1, XtPointer data2);
static void reject_invitation(Widget w, XtPointer data1, XtPointer data2);

/* Add dialog buttons callbacks */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data);
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data);
void add_cancel(Widget w, XtPointer app_data, XtPointer widget_data);

/* Mrm create callbacks */
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data);
void listform_created(Widget w, XtPointer app_data, XtPointer widget_data);
void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data);
void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data);
void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data);
void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data);

extern DveError dveError;


/* 
 * Mrm variables 
 */
const String  file_names[2] = { "/main.uid", "/add.uid"};     
String exact_file_names[2];
MrmHierarchy   hierarchy_id;     
MrmType        main_window_class, add_panel_class;

/* main panel widgets */
Widget         main_window, listform, statuslabel, add_but, accept_but, 
               reject_but, show_but, hide_but, info_but;

/* add dialog widgets */
Widget         users_list, activeusr_list, add_panel, user_field, host_field;

static MrmRegisterArg cb_list[] = 
{
  { "add_member",            (XtPointer)add_member            },
  { "accept_invitation",     (XtPointer)accept_invitation     },
  { "reject_invitation",     (XtPointer)reject_invitation     },
  { "show_plugin",           (XtPointer)show_plugin           },
  { "hide_plugin",           (XtPointer)hide_plugin           },
  { "info_plugin",           (XtPointer)info_plugin           },
  { "add_usr",               (XtPointer)add_usr               },
  { "delete_usr",            (XtPointer)delete_usr            },
  { "invite_usrs",           (XtPointer)invite_usrs           },
  { "add_cancel",            (XtPointer)add_cancel            },
  { "statuslabel_created",   (XtPointer)statuslabel_created   },
  { "listform_created",      (XtPointer)listform_created      },
  { "add_but_created",       (XtPointer)add_but_created       },
  { "accept_but_created",    (XtPointer)accept_but_created    },
  { "reject_but_created",    (XtPointer)reject_but_created    },
  { "show_but_created",      (XtPointer)show_but_created      },
  { "hide_but_created",      (XtPointer)hide_but_created      },
  { "info_but_created",      (XtPointer)info_but_created      },
  { "users_list_created",    (XtPointer)users_list_created    },
  { "activeusr_list_created",(XtPointer)activeusr_list_created},
  { "user_field_created",    (XtPointer)user_field_created    },
  { "host_field_created",    (XtPointer)host_field_created    }
};


/***********************************************************************
 *
 * Implementations of Netscape's plugin API functions
 *
 ***********************************************************************/

char *
NPP_GetMIMEDescription(void)
{
#ifdef DISP
  printf( "OpenDVE plugin:MIME description loaded\n");  
#endif
  return( MIME_EXT_DESCRIPTION );
}

NPError
NPP_Initialize(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Initialized\n");  
#endif
  return NPERR_NO_ERROR;
}

void
NPP_Shutdown(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Shutdown called\n");  
#endif
}


NPError 
NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
        PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
		
	instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
	
	This = (PluginInstance*) instance->pdata;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called\n");
#endif
	if (This != NULL)
	{
   	        int i;
		This->netscape_widget = NULL;
		This->display = NULL;
		This->mode = NP_FULL;   
		This->width = 400;
		This->height = 500;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called Mode=%d\n", This->mode);  
#endif

		/*--- Start custom code */
		
		information = NULL;
		create_recipient_list=0;
		appData.numInv=0;
		appData.numAcc=0;
		appData.numRej=0;
		appData.memDveAccList=NULL;
		appData.initPath=NULL;
		appData.helpPath=NULL;
		mbrs=NULL;

		This->len = 0;
		This->timerId = 0;

		/*--- End custom code */
		
		return NPERR_NO_ERROR;
	}
	else
		return NPERR_OUT_OF_MEMORY_ERROR;
}/* NPP_New */


NPError 
NPP_Destroy(NPP instance, NPSavedData** save)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

#ifdef DISP
	printf( "OpenDVE: NPP_Destroy called\n");  
#endif
        dveConfMgrLeaveConference(appData.session);
	free(mbrs);
	free(localURL);
	dvePluginMgrClose();
	QuitPlugin(appData.session);
	QuitPlugin(appData.privateSession);

	if (This != NULL) {

		/*--- Start custom code */

		/* Stream is closed, we have the data, so start a timer to
		 * call the draw routine
		 */

		if(This->timerId)
		    XtRemoveTimeOut(This->timerId);
		This->timerId = 0;

		/*--- End custom code */

		NPN_MemFree(instance->pdata);
		instance->pdata = NULL;
	}

	return NPERR_NO_ERROR;
}/* NPP_Destroy */

NPError 
NPP_SetWindow(NPP instance, NPWindow* window)
{
	PluginInstance* This;
	char *env;
	int i;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;
#ifdef DISP	
	printf( "OpenDVE: NPP_SetWindow called for %d\n", (int)instance);  
#endif

	This->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
	This->netscape_widget = XtWindowToWidget(This->display, (Window)window->window);
	This->width = window->width;
	This->height = window->height;
	
	This->appContext=XtWidgetToApplicationContext(This->netscape_widget);
	AppContext = This->appContext;
	parent=This->netscape_widget;
	

	/* Mrm operations */
	MrmInitialize();     
	env = getenv("HOME");
	for(i=0; i<2; i++)
	  {
	    exact_file_names[i] = malloc(100);
	    strcpy(exact_file_names[i], env);
	    strcat(exact_file_names[i], file_names[i]);
	  }

	MrmOpenHierarchyPerDisplay(XtDisplay(parent), 
				   2, 
				   exact_file_names,              
				   (MrmOsOpenParamPtr *) NULL, 
				   &hierarchy_id);

	if (MrmRegisterNames(cb_list, 
			     (MrmCount)XtNumber(cb_list)) != MrmSUCCESS)
	  {
	    fprintf(stderr, "Unable to register callbacks\n");
	    fflush(stderr);
	  }
	
	if (MrmFetchWidget(hierarchy_id, "simpleWindow",
			   parent, &main_window, &main_window_class)
	    != MrmSUCCESS) 
	  {
	    fprintf(stderr, "Unable to fetch top-level widget.\n");
	    fflush(stderr);
	  }      

	XtManageChild(main_window);
	XtRealizeWidget(parent);

	return NPERR_NO_ERROR;

} /* NPP_SetWindow */


NPError 
NPP_NewStream(NPP instance,
	      NPMIMEType type,
	      NPStream *stream, 
	      NPBool seekable,
	      uint16 *stype)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

/*--- Start custom code */
#ifdef DISP
	printf( "OpenDVE: NPP_NewStream called\n");  
#endif

	*stype = NP_NORMAL;
	
	localURL = changeURL(stream->url);
	
/*--- End custom code */
	return NPERR_NO_ERROR;
}/* NPP_NewStream */


int32 
NPP_WriteReady(NPP instance, NPStream *stream)
{
  
	PluginInstance* This;
	if (instance != NULL)
		This = (PluginInstance*) instance->pdata;
	
#ifdef DISP
	printf( "OpenDVE: NPP_WriteReady returns %d\n", req);  
#endif

	return 1024; 
	
}


int32 
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	int i;
	PluginInstance* This;
	

	if (instance != NULL)
	{
		This = (PluginInstance*) instance->pdata;
	      }

	information = XtRealloc(information, This->len + len + 2);

	strncpy(&(information[This->len]), buffer, len);

	This->len += len;

	return len;		/* The number of bytes accepted */
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	PluginInstance* This;
	XCharStruct bounds;
	int direction, ascent, descent;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	This = (PluginInstance*) instance->pdata;
	

	/*
	 * adding timer for the main initialization procedure 
	 */
	This->timerId = XtAppAddTimeOut(This->appContext,
					100,
					start_conference,
					(XtPointer)instance); 

	return NPERR_NO_ERROR;
}


void 
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
}

void 
NPP_Print(NPP instance, NPPrint* printInfo)
{
}


/*********************************************************************
 *
 * Mrm create callbacks
 *
 *********************************************************************/
void statuslabel_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  statuslabel = w;
  SetMessage("Initializing Conference Engine...");
}

void listform_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  listform = w;
}

void add_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  add_but = w;
}

void accept_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  accept_but = w;
  XtSetSensitive(w, False);
}

void reject_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  reject_but = w;
  XtSetSensitive(w, False);
}

void show_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  show_but = w;
  XtSetSensitive(w, False);
}
void hide_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  hide_but = w;
  XtSetSensitive(w, False);
}


void info_but_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  info_but = w;
  XtSetSensitive(w, False);
}

void users_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  users_list = w;
}

void activeusr_list_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  activeusr_list = w;
}

void user_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  user_field = w;
}

void host_field_created(Widget w, XtPointer app_data, XtPointer widget_data)
{
  host_field = w;
}



/*************************************************************************
  *
  *  Callbacks invoked by buttons
  *
  ************************************************************************/
showMessage(Widget w, char *msg)
{
  Widget dialog;
  XmString text;
  XmString title;

  text = XmStringCreateLtoR(msg, XmSTRING_DEFAULT_CHARSET);
  title = XmStringCreateSimple("Message");
  dialog = XmCreateMessageDialog(w, "message", NULL, 0);
  XtVaSetValues(dialog,
		XmNmessageString, text,
		XmNdialogType, XmDIALOG_INFORMATION,
		XmNdialogTitle, title,
		NULL);
  XmStringFree(text);
  XmStringFree(title);

  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));

  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
}


/***********************************
 * 
 * Add member dialog
 * 
 **********************************/
void
add_member(Widget w, XtPointer data1, XtPointer data2)
{
  Widget usr_lab, host_lab;

  mainWin = XtVaCreatePopupShell( "mainWin",
				 xmDialogShellWidgetClass,
				 parent,
				 XmNx, 370,
				 XmNy, 300,
				 XmNwidth, 500,
				 XmNheight, 470,
				 XmNtitle, "Add members",
				 NULL );

  

  if (MrmFetchWidget(hierarchy_id, "addWindow",
		     mainWin, &add_panel, &add_panel_class)
      != MrmSUCCESS) 
    {
      fprintf(stderr, "Unable to fetch top-level 'add' widget.\n");
      fflush(stderr);
    }      

  XtManageChild(add_panel);

  XtRealizeWidget(mainWin);

  XtPopup(mainWin, XtGrabNone);

  parseInformation(users_list, information);

  return;
}

/*
 * Add member dialog callbacks 
 */

/* Cancel callback */
void
add_cancel(Widget w, XtPointer app_data, XtPointer widget_data)
{
  XtDestroyWidget(mainWin);
}

/* Activate user (add button) */
void add_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;
  char *user;
  char *host;
  char malpa = '@';
  char addr[200];
  XmString address;

  XtVaGetValues(users_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(users_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      if(!XmListItemExists(activeusr_list, sel_item[0]))
	XmListAddItem(activeusr_list, sel_item[0], 0);
    }

  XtVaGetValues(user_field,
		XmNvalue, &user,
		NULL);
  XtVaGetValues(host_field,
		XmNvalue, &host,
		NULL);
  if((strlen(user) > 0) && (strlen(host) > 0))
    {
      strcpy(addr, user);
      strcat(addr, &malpa);
      strcpy(&addr[strlen(user) + 1], host);
      address = XmStringCreateSimple(addr);
      if(!XmListItemExists(activeusr_list, address))
	{
	  XmListAddItem(activeusr_list, address, 0);
	  XmTextFieldSetString(user_field, "");
	  XmTextFieldSetString(host_field, "");
	}
      XmStringFree(address);
    }
  
}
		
/* Unactivate user (delete button) */
void delete_usr(Widget w, XtPointer app_data, XtPointer widget_data)
{
  int sel_no;
  XmStringTable sel_item;

  XtVaGetValues(activeusr_list,
		XmNselectedItemCount, &sel_no,
		NULL);
  if(sel_no > 0)
    {
      XtVaGetValues(activeusr_list,
		    XmNselectedItems, &sel_item,
		    NULL);
      XmListDeleteItem(activeusr_list, sel_item[0]);
    }  
}

/* Invite active users */
void invite_usrs(Widget w, XtPointer app_data, XtPointer widget_data)
{
  DveConfMgrUserAddr addr;
  int item_no;
  XmStringTable items;
  char *item;
  int i;

  XtVaGetValues(activeusr_list,
		XmNitemCount, &item_no,
		NULL);
  if(item_no > 0)
    {
      XtVaGetValues(activeusr_list,
		    XmNitems, &items,
		    NULL);
      for(i=0; i<item_no; i++)
	{
	  XmStringGetLtoR(items[i], XmSTRING_DEFAULT_CHARSET, &item);
	  addr.userName = malloc(100);
	  addr.hostName = malloc(100);
	  addr.addrs = NULL;
	  parseIPaddress(item, addr.userName, addr.hostName);
	  XtFree(item);
	  appData.numInv = 1;
	  if(conferenceExist)
	    {
	      dveConfMgrAddMember(appData.session, &addr);
	    }
	  else
	    {
	      dveConfMgrInitiateConference(appData.session, &addr, 1);
	      conferenceExist = True;
	    }
	  XtFree(addr.userName);
	  XtFree(addr.hostName);
	}  
    }
  XtDestroyWidget(mainWin);
}


/********************* Add members dialog END ***************************/


/************************************************************************
 *
 * Callbacks regarding plugins management 
 * 
 ************************************************************************/
static void
info_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  char text[200];

  if (appData.currPlug >= 0)
    {
      sprintf(text, "Name:\t%s\nTitle:\t%s\nPort:\t%d\nProc ID:\t%d\nPath:\t%s\n", dvePluginList[appData.currPlug]->name, 
	      dvePluginList[appData.currPlug]->title,
	      dvePluginList[appData.currPlug]->port,
	      dvePluginList[appData.currPlug]->processID,
	      dvePluginList[appData.currPlug]->basePath);
      showMessage(parent, text);
    }
  return;
}

static void
show_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  static JBPluginManagementArg dane;

  dane.port=dvePluginList[appData.currPlug]->port;
  SendPluginMessageGeneric(appData.session, JB_CONF_MANAGER_PRIVATE,
			   JB_CONF_MANAGER_PRIVATE,
			   dvePluginManagementMsgTagShow,
			   -1, &dane, sizeof(dane));
  
  if ((dvePluginList[appData.currPlug]->processID) <=0)
  {
    dvePluginMgrStartPlugin(appData.session,
	   		    dvePluginList[appData.currPlug],
			    PLUGIN_MAPPED);
  }
  else
    {
      printf("Selected plugin is already active!");
      fflush(stdout);
    }
  
  return;
}

static void
hide_plugin(Widget w, XtPointer data1, XtPointer data2)
{
  static JBPluginManagementArg dane;

  dane.port=dvePluginList[appData.currPlug]->port;
  SendPluginMessageGeneric(appData.session, JB_CONF_MANAGER_PRIVATE,
			   JB_CONF_MANAGER_PRIVATE,
			   dvePluginManagementMsgTagQuit,
			   -1, &dane, sizeof(dane));
  
  if ((dvePluginList[appData.currPlug]->processID) >0)
  {
    dvePluginMgrQuitPlugin(appData.session,
	   		    dvePluginList[appData.currPlug]);
  }
  else
    {
      printf("Selected plugin is not running!");
      fflush(stdout);
    }
  
  return;
}

void
accListCallback(DveMotifRecipientList list,
		DveMotifRecipientListCallbackStruct* data)
{
  return;
}

static void
accept_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  dveConfMgrAcceptInvitation(appData.session);
  SetMessage("Invitation to teleconference has been accepted!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
  create_recipient_list=1;
    
  return;
}

static void
reject_invitation(Widget w, XtPointer data1, XtPointer data2)
{
  dveConfMgrRejectInvitation(appData.session);
  SetMessage("Invitation to teleconference has been rejected!");
  XtSetSensitive(accept_but, False);
  XtSetSensitive(reject_but, False);
  return;
}


/***************************************************************************
 * 
 * DVE Events callbacks
 * 
 ***************************************************************************/
static void
partyAccepted(DveSession dveSession, DveEvent* event)
{
  /*  fprintf(stdout,"user: %s on host: %s accepted invitation\n", event->arg.memberAccepted.member.userName, event->arg.memberAccepted.member.hostName);*/
    
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);

  appData.numAcc+=1;

  return;
}

static void   
partyRejected(DveSession dveSession, DveEvent* event)
{
  char text[100];

  sprintf(text, "User %s@%s\n rejected invitation!\n", 
	  event->arg.memberRejected.member.userName, 
	  event->arg.memberRejected.member.hostName);
  showMessage(parent, text);

  /*  fprintf(stdout,"user: %s on host: %s rejected invitation\n", 
	  event->arg.memberRejected.member.userName, 
	  event->arg.memberRejected.member.hostName);
  fflush(stdout);
  */
  return;
}

static void
partyLeft(DveSession dveSession, DveEvent* event)
{
/*  fprintf(stdout,"user: %s on host: %s left the conference\n", event->arg.memberLeft.member.userName, event->arg.memberLeft.member.hostName);
  fflush(stdout);*/
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
  return;
}

static void
partyNew(DveSession dveSession, DveEvent* event)
{
  /*  fprintf(stdout,"user: %s on host: %s\nhas been invited to join teleconference\n", event->arg.newMember.member.userName, event->arg.newMember.member.hostName); */
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
  fflush(stdout);
  return;
}

static void
changedDND(DveSession dveSession, DveEvent* event)
{
  fprintf(stdout,"Local DND flag is set to %s\n", (event->arg.DNDChanged.DND)?"True":"False");
  fflush(stdout);
  return;
}

static void
SetMessage(const char *msg)
{
  XmString str=XmStringCreateSimple((char*)msg);
  XtVaSetValues(statuslabel,
		XmNlabelString, str,
		NULL);
  XmStringFree(str);
  return;
}

/***************************************************************************

  Definitions of OpenDVE-specific functions

***************************************************************************/  
int initLocalOpenDVE()
{
  
  if ((err=check_process("ISregistrar", ""))<0)
  {
    failure("ISregistrar problem\n");
    return err;
  }
  
  if ((err=check_process("ISconferenceEngine", ""))<0)
  {
    failure("ISconferenceEngine problem\n");
    return err;
  }
 
  return 0;
}

int connectDVE()
{
    if ((err=introduceYourself(&(appData.session),
			       &(appData.privateSession)))<0)
    {
        SetMessage("Can not connect to Conference Engine... Check and try again...");
        initLocalOpenDVE();
	return err;   
    }   
    SetMessage("Connected to Conference Engine...");
    fflush(stdout);
    SetMessage("Initializing conferencing panel...");
    if ((err=init_plugins(appData.session))<0)
    { 
        SetMessage("Can not initialize conferencing panel(plugins)... Aborting...");
        return err;
    }

    if ((err=init_members(appData.session))<0)
      {
	SetMessage("Can not initialize members... Aborting...");
	return;
      }

    DvePartListClrAll(&(appData.memAccList));
    appData.memDveAccList = dveMotifRecipientListCreate(appData.session,
							membersAccList,
							&(appData.memAccList),
							False);
    dveMotifRecipientListSetCallback(appData.memDveAccList, accListCallback);

    SetMessage("Conferencing panel ready to use.");
    appData.initiatorHandle=appData.localHandle;

    return 0;
}

DveConfMgrInvitationCallback invited(DveSession dveSession,
				     DveConfMgrInvitedArg* arg)
{
  char text[100];
  int res;

  sprintf(text, "RECEIVED INVITATION!\n from: %s@%s\n", 
	  (arg->parts[arg->initiatorIndex]).userName, 
	  (arg->parts[arg->initiatorIndex]).hostName);
  showMessage(parent, text);

  appData.numInv = arg->numParts;
  appData.initiatorHandle=(arg->parts[arg->initiatorIndex]).userHandle;

  XtSetSensitive(accept_but, True);
  XtSetSensitive(reject_but, True);
  SetMessage("Press ACCEPT or REJECT");
  return;
}

void start_conference(void)
{ 

  int err;
 
  /* starting local ISregistrar and ISconferenceEngine if necessary */
  SetMessage("Initializing Conference Engine...");

  if ((err=initLocalOpenDVE())<0)
    {
      SetMessage("Can not initialize OpenDVE... Aborted...");
      return;
    }

  /* connect to DVE */
  if ((err=connectDVE())<0)
    {
      connectDVE();
    }
   
}

void exit_conference(void)
{
  dveConfMgrLeaveConference(appData.session);
  QuitPlugin(appData.session);
}

void started(DveSession dveSession, DveEvent* event)
{
  int ilosc, i;

  ilosc=event->arg.starting.numMembers;
  appData.state=event->arg.starting.state;
  /*  fprintf(stdout, "I am invited! %d %d\n", ilosc, (int)appData.state);
      for (i=0;i<ilosc;i++)
    fprintf(stdout, "%s@%s\n", event->arg.starting.members[i].part.userName, event->arg.starting.members[i].part.hostName);
  fflush(stdout);*/

  dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
    
  return;
}


QuitPlugin(DveSession dveSession)
{
 
   dveUnregisterWithXt(dveSession);
   dveCloseSession(dveSession);
   return;
}

/*
 * QuitPlugin if requested.
 */

static void
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
  static JBPluginManagementArg dane;

  memcpy(&dane, packet->data, sizeof(JBPluginManagementArg));
  switch ((DvePluginManagementMsgTag)packet->tag)
   {
     case dvePluginManagementMsgTagShow:
         dvePluginMgrStartPlugin(appData.session,
	   		    dvePluginMgrLocatePluginByPortNumber(dane.port),
			    PLUGIN_MAPPED);
	 XtSetSensitive(info_but, True);
	 XtSetSensitive(show_but, True);
	 XtSetSensitive(hide_but, True);
         break;
     case dvePluginManagementMsgTagQuit:
         dvePluginMgrQuitPlugin(appData.session,
	   		    dvePluginMgrLocatePluginByPortNumber(dane.port));
         break;
   }
}

int
introduceYourself(DveSession *session, DveSession *privatesession)
{
   /*
    * Start up OpenDVE.
    */
   if (!((*session)=dveOpenSession(JB_CONF_MANAGER_PORT)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}
       return -1;
   }
   if (!((*privatesession)=dveOpenSession(JB_CONF_MANAGER_PRIVATE)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}
       return -1;
   }

   dveSetUserCallback(*privatesession, JB_CONF_MANAGER_PRIVATE, PluginControlMsg);
   dveSetEventCallback(*session,dveEventTagStarting,started);
   dveSetEventCallback(*session, dveEventTagMemberAccepted, partyAccepted);
   dveSetEventCallback(*session, dveEventTagMemberRejected, partyRejected);
   dveSetEventCallback(*session, dveEventTagMemberLeft, partyLeft);
   dveSetEventCallback(*session, dveEventTagNewMember, partyNew);
   dveSetEventCallback(*session, dveEventTagDNDChanged, changedDND);
   dveConfMgrSetInvitationCallback(*session, invited);
     
   /*
    * Register this event processing with the Xt toolkit.  Without this
    * call OpenDVE events will not be received.
    */
   if (dveRegisterWithXt(*session,AppContext)<0)
   {
      fprintf(stderr, "Unable to register with Xt loop.");
      dveCloseSession(*session);
      return -1;
   }
   if (dveRegisterWithXt(*privatesession,AppContext)<0)
   {
      fprintf(stderr, "Unable to register with Xt loop.");
      dveCloseSession(*session);
      return -1;
   }

   dveReportPresence(*session);
   dveReportPresence(*privatesession);
   appData.localHandle = dveGetLocalUserHandle(*session);
   
   return 0;
}

void
pluginSelected(Widget w, XtPointer data1, XtPointer data2)
{
  XmListCallbackStruct *cbs=(XmListCallbackStruct *)data2;

/*  printf("Selected item: %d\n", cbs->item_position-1);
  printf("Number of items: %d\n", cbs->selected_item_count);
  fflush(stdout);*/
  if (cbs->selected_item_count > 0)
  {
    appData.currPlug = cbs->item_position-1;
    XtSetSensitive(show_but, True);
    XtSetSensitive(hide_but, True);
    XtSetSensitive(info_but, True);
  }
  else
  {
    appData.currPlug = -1;
    XtSetSensitive(show_but, False);
    XtSetSensitive(hide_but, False);
    XtSetSensitive(info_but, False);
  }
  
  return;
}

void
createPluginsInterface()
{
  int i;
  XmString label;

  label=XmStringCreateLocalized("Available plugins:");
  pluginsLabel=XtVaCreateManagedWidget("pluginsLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 10,
				       XmNy, 10,
				       NULL);


  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 10); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 220); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  pluginsList = XmCreateScrolledList(listform, "Plugins", wArg, numArg);

  plugTlist=NULL;
  XtVaSetValues(pluginsList,
	        XmNitems, plugTlist,
		XmNitemCount, 0,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
					   		
  plugTlist=(XmStringTable)XtMalloc(appData.numPlug*sizeof(XmString *));
  for(i=0; i<appData.numPlug;i++)
  {
    plugTlist[i]=XmStringCreateLocalized(dvePluginList[i]->title);
  }
  
  XtVaSetValues(pluginsList,
                XmNitems, plugTlist,
		XmNitemCount, appData.numPlug,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
  XtAddCallback(pluginsList, XmNsingleSelectionCallback, pluginSelected, NULL);
  XtManageChild(pluginsList);
  
  for(i=0; i<appData.numPlug;i++)
  {
    XmStringFree(plugTlist[i]);
  }
  XtFree(plugTlist);
  
  return;
}

int
init_plugins(DveSession session)
{
  char *home;
  
  appData.initPath=(char*)XtMalloc(256);  
  if (appData.initPath == NULL)
  {
     printf("Can not allocate memory for string! Exiting...\n");
     return -1;
   }

  home = getenv("HOME");
  strcpy(appData.initPath, home);
  strcat(appData.initPath, "/plugins.cfg");
  
  /* HELP VIEWER not used since InSoft's Help library is missing 

     appData.helpPath=(char*)XtMalloc(256);  
  if (appData.helpPath == NULL)
  {
     printf("Can not allocate memory for string! Exiting...\n");
     return -1;
   }
  appData.helpPath="";
  appData.helpPath=getenv("HOME");
  strcat(appData.helpPath, "/DVE/localplugins");
  err=IS_AddHelpPath(session, appData.helpPath);
  printf("Result form addHelpPAth: %d\n", err);
  fflush(stdout);*/

  
  /*  printf("Plugins are searched at: %s\n", appData.initPath);
  fflush(stdout);*/

  err = dvePluginMgrInitialize(session, appData.initPath, PluginReady, PluginDone);

  if (err < 0)
    {
      fprintf(stderr, "Error %d on dvePluginMgrInitialize().\n", 
	      dvePluginMgrErrno);
      return -1;
    }
  XtFree(appData.initPath);
  
  appData.numPlug = dvePluginMgrIdentifyAvailablePlugins(session);
  createPluginsInterface();
  return 0;
}

void
PluginReady(pluginObject plugin)
{
  return;
}

void
PluginDone(pluginObject plugin)
{
  return;
}

void
membersAccSelected(Widget w, XtPointer data1, XtPointer data2)
{
  return;
}

void
membersRejSelected(Widget w, XtPointer data1, XtPointer data2)
{
  return;
}

void
createMembersInterface()
{
  int i;

  /* list of members which accepted invitation */
  label=XmStringCreateLocalized("Active members:");
  membersLabel=XtVaCreateManagedWidget("membersLabel",
				       xmLabelGadgetClass,
				       listform,
				       XmNlabelString, label,
				       XmNx, 240,
				       XmNy, 10,
				       NULL);

  
  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 240); numArg++;
  XtSetArg(wArg[numArg], XmNy, 30); numArg++;
  XtSetArg(wArg[numArg], XmNwidth, 250); numArg++;
  XtSetArg(wArg[numArg], XmNlistSizePolicy, XmCONSTANT); numArg++;
  membersAccList = XmCreateScrolledList(listform, "membersAcc",
					       wArg, numArg);

  XtVaSetValues(membersAccList,
		XmNitemCount, 0/*appData.numInv*/,
		XmNvisibleItemCount, 6,
		XmNselectionPolicy, XmMULTIPLE_SELECT,
		NULL);
					   		
  XtManageChild(membersAccList);
  
}

int
init_members(DveSession session)
{
  createMembersInterface();
  return 0;
}


/*
 * Passes data from this plugin to other conferenced members.
 */
int
SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size)
{
   DveDataPacket *packet;
   int returnCode,i;

/*   printf("Size of data to send: %d\n", size);
   fflush(stdout);
  */ 
   packet = (DveDataPacket*) dveMallocDataPacket(dveSession, size);
   packet->tag = (u_int32)messageType;
   packet->pluginProtocol = protocol;
   packet->destPort = port;

/*   printf("SendGeneric to port %d\n", packet->destPort);*/
   DvePartListClrAll(&(packet->destParts));

   if (dest < 0)
   {
      DvePartListSetAll(&(packet->destParts));
   }
   else
   {
      DvePartListSet(dest,&(packet->destParts));
   }

   if (size > 0)
      memcpy(packet->data, data, size);

   returnCode = dveSendDataPacket(dveSession, packet);

   dveFreeDataPacket(packet);

/*   printf("Result from Send(): %d\n", returnCode);*/
   fflush(stdout);

   return(returnCode);
}


/*
 * function adding members from http file to adding list
 */
void parseInformation(Widget list, char *info)
{
  int pos = 0;
  char user[100];
  XmString xmstring;
  int number, i;

  sscanf(info, "%d\n", &number);
  sscanf(info, "%s\n", user);
  pos = strlen(user) + 1;
  for(i=0; i<number; i++)
    {
      sscanf(&info[pos], "%s\n", user);
      pos += strlen(user) + 1;
      xmstring = XmStringCreateSimple(user);
      XmListAddItem(list, xmstring, 0);
      XmStringFree(xmstring);
    }

}












